package net.knarcraft.knargui.item;

import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.function.Consumer;

/**
 * A factory class for creating GUI items
 */
@SuppressWarnings("unused")
public interface GUIItemFactory {

    /**
     * Builds the item as described in previous calls
     *
     * @return <p>The output of this factory</p>
     */
    @NotNull
    ItemStack build();

    /**
     * Sets the name of this factory's item
     *
     * @param name <p>The new item name</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    @NotNull
    GUIItemFactory setName(@NotNull String name);

    /**
     * Sets the lore for this factory's item
     *
     * @param lore <p>The new lore</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    @NotNull
    GUIItemFactory setLore(@NotNull List<String> lore);

    /**
     * Sets whether this factory's item should look enchanted
     *
     * @param enchanted <p>True if the item should look enchanted</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    @NotNull
    GUIItemFactory setEnchanted(boolean enchanted);

    /**
     * Changes some metadata for this factory's item
     *
     * @param action <p>The action to be run on the metadata</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    @NotNull
    GUIItemFactory changeItemMeta(@NotNull Consumer<ItemMeta> action);

    /**
     * Changes some metadata for this factory's item
     *
     * @param metaType <p>The type of metadata to change</p>
     * @param action   <p>The action to be run on the metadata</p>
     * @param <M>      <p>The type of the metadata</p>
     * @return <p>The factory. Used for chaining commands</p>
     */
    @NotNull <M extends ItemMeta> GUIItemFactory changeItemMeta(@NotNull Class<M> metaType, @NotNull Consumer<M> action);

    /**
     * Throws an exception if this factory's item has null metadata
     *
     * @return <p>The metadata for this factory's item</p>
     */
    @NotNull <M extends ItemMeta> M checkForNullMetadata(@NotNull Class<M> metaType);

}
